﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace WPFDisplayASimpleMap
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private EditInteractiveOverlay editOverlay;
        private Collection<Feature> FeatureObjects;
        private InMemoryFeatureLayer myFeatureLayer;
        private LayerOverlay myFeatureLayerOverlay;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void CreateFeatureObjectLayer()
        {
            //Get features from data source
            //FeatureObjects = _featureObjectCasheProxy.GetAll().ToList();
            //NotifyOfPropertyChange(() => FeatureObjects);

            ShapeFileFeatureLayer shpLayer = new ShapeFileFeatureLayer("../../App_Data/USStates.shp");
            shpLayer.Open();
            FeatureObjects = shpLayer.FeatureSource.GetAllFeatures(ReturningColumnsType.AllColumns);

            FeatureCache featureCache = new FeatureCache();

            try
            {
                //Add my feature layer
                myFeatureLayer = new InMemoryFeatureLayer();
                myFeatureLayer.Open();

                if (!myFeatureLayer.FeatureSource.IsOpen)
                    myFeatureLayer.FeatureSource.Open();

                //myFeatureLayer.Columns.Add(new FeatureSourceColumn("FeatureObjectID"));
                //myFeatureLayer.Columns.Add(new FeatureSourceColumn("Code"));
                //myFeatureLayer.Columns.Add(new FeatureSourceColumn("Name"));
                myFeatureLayer.EditTools.BeginTransaction();
                //myFeatureDataItems = new BindableCollection();

                foreach (var featureObject in FeatureObjects)
                {
                    var myFeatureObject = new Feature(featureObject.GetWellKnownBinary());

                    //TODO: Assign attribute data here
                    //myFeatureObject.ColumnValues["FeatureObjectID"] = featureObject.DestinationID.ToString();
                    //myFeatureObject.ColumnValues["Code"] = featureObject.DestinationCode;
                    //myFeatureObject.ColumnValues["Name"] = featureObject.DestinationName;

                    myFeatureLayer.EditTools.Add(myFeatureObject.GetShape(), myFeatureObject.ColumnValues);

                    //Add to bindable collection for grid
                    //myFeatureDataItems.Add(featureObject);
                }

                myFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.SimpleColors.BrightBlue, GeoColor.SimpleColors.PaleRed, 2);
                myFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

                //myFeatureLayer.FeatureSource.Projection = proj4Projection;
                myFeatureLayer.FeatureSource.GeoCache = featureCache;
                myFeatureLayer.EditTools.CommitTransaction();
                myFeatureLayer.Close();

                if (wpfMap1.Overlays.Contains("FeatureObjectLayerOverlay"))
                {
                    myFeatureLayerOverlay = wpfMap1.Overlays["FeatureObjectLayerOverlay"] as LayerOverlay;
                    if (myFeatureLayerOverlay != null)
                    {
                        myFeatureLayerOverlay.Layers.Clear();
                        myFeatureLayerOverlay.Layers.Add(myFeatureLayer);
                        myFeatureLayerOverlay.Refresh();
                    }
                }
                else
                {
                    myFeatureLayerOverlay = new LayerOverlay();
                    myFeatureLayerOverlay.Layers.Add(myFeatureLayer);
                }
            }
            catch (Exception ex)
            {
            }
        }

        private void EditMapFeature(Feature feature, bool zoomToExtent)
        {
            //Create edit layer
            editOverlay = wpfMap1.EditOverlay;
            editOverlay.Name = "EditInteractiveOverlay";

            wpfMap1.EditOverlay = editOverlay;
            wpfMap1.EditOverlay.CanAddVertex = true;
            wpfMap1.EditOverlay.CanDrag = true;
            wpfMap1.EditOverlay.CanRemoveVertex = true;
            wpfMap1.EditOverlay.CanReshape = true;
            wpfMap1.EditOverlay.CanResize = true;
            wpfMap1.EditOverlay.CanRotate = true;

            try
            {
                if (feature != null)
                {
                    wpfMap1.EditOverlay.EditShapesLayer.InternalFeatures.Add(feature);
                    wpfMap1.EditOverlay.CalculateAllControlPoints();
                    if (!wpfMap1.EditOverlay.EditShapesLayer.IsOpen)
                        wpfMap1.EditOverlay.EditShapesLayer.Open();

                    //Clear highlighted layer I do this so the highlighted layer doesn't appear over the edit layer
                    //LayerOverlay highlightOverlay = (LayerOverlay)wpfMap1.Overlays["HighlightOverlay"];
                    //InMemoryFeatureLayer highlightLayer = (InMemoryFeatureLayer)highlightOverlay.Layers["HighlightLayer"];

                    //if (highlightLayer != null)
                    //{
                    //    highlightLayer.Open();
                    //    highlightLayer.InternalFeatures.Clear();
                    //    highlightLayer.Close();
                    //    highlightOverlay.IsVisible = false;
                    //    highlightOverlay.Refresh();
                    //}
                }
                else
                {
                    wpfMap1.EditOverlay.EditShapesLayer.InternalFeatures.Clear();
                }

                if (zoomToExtent)
                {
                    RectangleShape extent = wpfMap1.EditOverlay.EditShapesLayer.GetBoundingBox();
                    wpfMap1.CurrentExtent = extent;
                }

                //ButtonUpdateIsVisible = true;
                //NotifyOfPropertyChange(() => ButtonUpdateIsVisible);

                //ButtonCancelIsVisible = true;
                //NotifyOfPropertyChange(() => ButtonCancelIsVisible);

                //ButtonSaveIsVisible = false;
                //NotifyOfPropertyChange(() => ButtonSaveIsVisible);

                //myFeatureLayerOverlay.Refresh();
                wpfMap1.Refresh();
            }
            catch (Exception ex)
            {
            }
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            wpfMap1.MapUnit = GeographyUnit.DecimalDegree;
            wpfMap1.CurrentExtent = new RectangleShape(-187.638921875, 93.86171875, -49.826421875, -4.75156250000001);

            LayerOverlay lo = new LayerOverlay();
            wpfMap1.Overlays.Add("FeatureObjectLayerOverlay", lo);

            CreateFeatureObjectLayer();

            wpfMap1.MapClick += wpfMap1_MapClick;

            wpfMap1.Refresh();
        }

        private void wpfMap1_MapClick(object sender, MapClickWpfMapEventArgs e)
        {
            myFeatureLayerOverlay = wpfMap1.Overlays["FeatureObjectLayerOverlay"] as LayerOverlay;
            myFeatureLayer = (InMemoryFeatureLayer)myFeatureLayerOverlay.Layers.FirstOrDefault();
            myFeatureLayer.Open();

            var feature = myFeatureLayer.QueryTools.GetFeaturesContaining(e.WorldLocation, ReturningColumnsType.AllColumns).FirstOrDefault();

            EditMapFeature(feature, false);
        }
    }
}